"""Result wrapper for Databricks SQL statement execution."""

import dataclasses
import threading
import time
from concurrent.futures import ThreadPoolExecutor, FIRST_COMPLETED, wait as concurrent_wait
from typing import Optional, Iterator, TYPE_CHECKING

import pyarrow as pa
import pyarrow.ipc as pipc

from .exceptions import SqlStatementError
from .types import column_info_to_arrow_field
from ...libs.databrickslib import databricks_sdk, WorkspaceClient, DatabricksDummyClass
from ...libs.pandaslib import PandasDataFrame
from ...libs.polarslib import polars
from ...libs.sparklib import SparkDataFrame
from ...pyutils.waiting_config import WaitingConfigArg, WaitingConfig
from ...requests.session import YGGSession
from ...types import spark_dataframe_to_arrow_table, \
    spark_schema_to_arrow_schema, arrow_table_to_spark_dataframe

if databricks_sdk is not None:
    from databricks.sdk.service.sql import (
        StatementState, StatementResponse, Disposition, StatementStatus
    )
else:
    StatementState = DatabricksDummyClass
    StatementResponse = DatabricksDummyClass
    Disposition = DatabricksDummyClass
    StatementStatus = DatabricksDummyClass


if TYPE_CHECKING:
    pass


DONE_STATES = {
    StatementState.CANCELED, StatementState.CLOSED, StatementState.FAILED,
    StatementState.SUCCEEDED
}

FAILED_STATES = {
    StatementState.FAILED, StatementState.CANCELED
}

__all__ = [
    "StatementResult"
]


@dataclasses.dataclass
class StatementResult:
    """Container for statement responses, data extraction, and conversions."""
    workspace_client: WorkspaceClient
    warehouse_id: str
    statement_id: str
    disposition: Disposition

    _response: Optional[StatementResponse] = dataclasses.field(default=None, repr=False)

    _spark_df: Optional[SparkDataFrame] = dataclasses.field(default=None, repr=False)
    _arrow_table: Optional[pa.Table] = dataclasses.field(default=None, repr=False)

    def __getstate__(self):
        """Serialize statement results, converting Spark dataframes to Arrow.

        Returns:
            A pickle-ready state dictionary.
        """
        state = self.__dict__.copy()

        _spark_df = state.pop("_spark_df", None)

        if _spark_df is not None:
            state["_arrow_table"] = spark_dataframe_to_arrow_table(_spark_df)

        return state

    def __setstate__(self, state):
        """Restore statement result state, rehydrating cached data.

        Args:
            state: Serialized state dictionary.
        """
        _spark_df = state.pop("_spark_df")

    def __iter__(self):
        """Iterate over Arrow record batches."""
        return self.to_arrow_batches()

    def __repr__(self):
        return "StatementResult(url='%s')" % self.monitoring_url

    def __str__(self):
        return self.monitoring_url

    @property
    def monitoring_url(self):
        return "%s/sql/warehouses/%s/monitoring?queryId=%s" % (
            self.workspace_client.config.host,
            self.warehouse_id,
            self.statement_id
        )

    @property
    def is_spark_sql(self):
        """Return True when this result was produced by Spark SQL."""
        return self._spark_df is not None

    @property
    def response(self):
        """Return the latest statement response, refreshing when needed.

        Returns:
            The current StatementResponse object.
        """
        if self.is_spark_sql:
            return StatementResponse(
                statement_id=self.statement_id or "sparksql",
                status=StatementStatus(
                    state=StatementState.SUCCEEDED
                )
            )
        elif not self.statement_id:
            return StatementResponse(
                statement_id="unknown",
                status=StatementStatus(
                    state=StatementState.PENDING
                )
            )

        statement_execution = self.workspace_client.statement_execution

        if self._response is None:
            # Initialize
            self._response = statement_execution.get_statement(self.statement_id)
        elif self._response.status.state not in DONE_STATES:
            # Refresh
            self._response = statement_execution.get_statement(self.statement_id)

        return self._response

    def api_result_data_at_index(self, chunk_index: int):
        """Fetch a specific result chunk by index.

        Args:
            chunk_index: Result chunk index to retrieve.

        Returns:
            The SDK result chunk response.
        """
        sdk = self.workspace_client

        return sdk.statement_execution.get_statement_result_chunk_n(
            statement_id=self.statement_id,
            chunk_index=chunk_index,
        )

    @property
    def status(self):
        """Return the statement status, handling persisted data.

        Returns:
            A StatementStatus object.
        """
        return self.response.status

    @property
    def state(self):
        """Return the statement state.

        Returns:
            The StatementState enum value.
        """
        return self.status.state

    @property
    def manifest(self):
        """Return the SQL result manifest, if available.

        Returns:
            The result manifest or None for Spark SQL results.
        """
        self.wait()
        return self.response.manifest

    @property
    def result(self):
        """Return the raw statement result object.

        Returns:
            The statement result payload from the API.
        """
        self.wait()
        return self.response.result

    @property
    def done(self):
        """Return True when the statement is in a terminal state.

        Returns:
            True if the statement is done, otherwise False.
        """
        return self.state in DONE_STATES

    @property
    def failed(self):
        """Return True when the statement failed or was cancelled.

        Returns:
            True if the statement failed or was cancelled.
        """
        return self.state in FAILED_STATES

    @property
    def persisted(self):
        """Return True when data is cached locally.

        Returns:
            True when cached Arrow or Spark data is present.
        """
        return self._spark_df is not None or self._arrow_table is not None

    def persist(self):
        """Cache the statement result locally as Arrow data.

        Returns:
            The current StatementResult instance.
        """
        if not self.persisted:
            self._arrow_table = self.to_arrow_table()
        return self

    def external_links(self):
        """Yield external result links for EXTERNAL_LINKS dispositions.

        Yields:
            External link objects in result order.
        """
        assert self.disposition == Disposition.EXTERNAL_LINKS, "Cannot get from %s, disposition %s != %s" % (
            self, self.disposition, Disposition.EXTERNAL_LINKS
        )

        result_data = self.result
        wsdk = self.workspace_client

        seen_chunk_indexes = set()

        while True:
            links = getattr(result_data, "external_links", None) or []
            if not links:
                return

            # yield all links in the current chunk/page
            for link in links:
                yield link

            # follow the next chunk (usually only present/meaningful on the last link)
            next_internal = getattr(links[-1], "next_chunk_internal_link", None)
            if not next_internal:
                return

            try:
                chunk_index = int(next_internal.rstrip("/").split("/")[-1])
            except Exception as e:
                raise ValueError(
                    f"Bad next_chunk_internal_link {next_internal!r}: {e}"
                )

            # cycle guard
            if chunk_index in seen_chunk_indexes:
                raise ValueError(
                    f"Detected chunk cycle at index {chunk_index} from {next_internal!r}"
                )
            seen_chunk_indexes.add(chunk_index)

            try:
                result_data = wsdk.statement_execution.get_statement_result_chunk_n(
                    statement_id=self.statement_id,
                    chunk_index=chunk_index,
                )
            except Exception as e:
                raise ValueError(
                    f"Cannot retrieve data batch from {next_internal!r}: {e}"
                )

    def raise_for_status(self):
        if self.failed:
            raise SqlStatementError.from_statement(self)
        return self

    def wait(
        self,
        wait: WaitingConfigArg = True,
        raise_error: bool = True
    ):
        """Wait for statement completion with optional timeout.

        Args:
            wait: Waiting config
            raise_error: Raise error if failed

        Returns:
            The current StatementResult instance.
        """
        wait = WaitingConfig.check_arg(wait)
        iteration, start = 0, time.time()

        if not self.done:
            wait.sleep(iteration=iteration, start=start)
            iteration += 1

        if raise_error:
            self.raise_for_status()

        return self

    def arrow_schema(self):
        """Return the Arrow schema for the result.

        Returns:
            An Arrow Schema instance.
        """
        if self.persisted:
            if self._arrow_table is not None:
                return self._arrow_table.schema
            elif self._spark_df is not None:
                return spark_schema_to_arrow_schema(self._spark_df.schema)
            else:
                raise NotImplementedError("")

        manifest = self.manifest

        metadata = {
            "source": "databricks-sql",
            "sid": self.statement_id or ""
        }

        if manifest is None:
            return pa.schema([], metadata=metadata)

        fields = [
            column_info_to_arrow_field(_) for _ in manifest.schema.columns
        ]

        return pa.schema(
            fields,
            metadata=metadata
        )

    def to_arrow_table(
        self,
        parallel_pool: int = 4
    ) -> pa.Table:
        """Collect the statement result into a single Arrow table.

        Args:
            parallel_pool: Maximum parallel fetch workers.

        Returns:
            An Arrow Table containing all rows.
        """
        if self.persisted:
            if self._arrow_table is not None:
                return self._arrow_table
            else:
                return self._spark_df.toArrow()

        batches = list(self.to_arrow_batches(parallel_pool=parallel_pool))

        if not batches:
            return pa.Table.from_batches([], schema=self.arrow_schema())

        return pa.Table.from_batches(batches)

    def to_arrow_batches(
        self,
        parallel_pool: int = 4,
        batch_size: Optional[int] = None
    ) -> Iterator[pa.RecordBatch]:
        """Stream the result as Arrow record batches.

        Args:
            parallel_pool: Maximum parallel fetch workers.
            batch_size: Fetch batch size

        Yields:
            Arrow RecordBatch objects.
        """
        if self.persisted:
            if self._arrow_table is not None:
                for batch in self._arrow_table.to_batches(max_chunksize=batch_size):
                    yield batch
            elif self._spark_df is not None:
                for batch in self._spark_df.toArrow().to_batches(max_chunksize=batch_size):
                    yield batch
            else:
                raise NotImplementedError("")
        else:
            _tls = threading.local()

            def _get_session():
                # requests.Session-style objects are not reliably thread-safe, so keep one per thread
                s = getattr(_tls, "session", None)
                if s is None:
                    s = YGGSession()
                    _tls.session = s
                return s

            def _fetch_bytes(link):
                s = _get_session()
                resp = s.get(link.external_link, verify=False, timeout=10)
                resp.raise_for_status()
                return resp.content

            max_workers = max(1, int(parallel_pool) if parallel_pool else 4)
            max_in_flight = max_workers * 2  # keeps pipeline full without exploding memory

            links_iter = enumerate(self.external_links())
            pending = {}  # future -> idx
            ready = {}  # idx -> bytes
            next_idx = 0

            def submit_more(ex):
                while len(pending) < max_in_flight:
                    try:
                        idx, link = next(links_iter)
                    except StopIteration:
                        break
                    fut = ex.submit(_fetch_bytes, link)
                    pending[fut] = idx

            with ThreadPoolExecutor(max_workers=max_workers) as ex:
                submit_more(ex)

                while pending:
                    done, _ = concurrent_wait(pending, return_when=FIRST_COMPLETED)

                    # collect completed downloads
                    for fut in done:
                        idx = pending.pop(fut)
                        ready[idx] = fut.result()  # raises here if the GET failed

                    # yield strictly in-order
                    while next_idx in ready:
                        content = ready.pop(next_idx)

                        buf = pa.BufferReader(content)

                        # IPC stream (your current format)
                        reader = pipc.open_stream(buf)

                        # if it’s IPC file instead:
                        # reader = pipc.open_file(buf)

                        for batch in reader:
                            yield batch

                        next_idx += 1

                    submit_more(ex)

    def to_pandas(
        self,
        parallel_pool: Optional[int] = 4
    ) -> PandasDataFrame:
        """Return the result as a pandas DataFrame.

        Args:
            parallel_pool: Maximum parallel fetch workers.

        Returns:
            A pandas DataFrame with the result rows.
        """
        return self.to_arrow_table(parallel_pool=parallel_pool).to_pandas()

    def to_polars(
        self,
        parallel_pool: int = 4
    ) -> "polars.DataFrame":
        """Return the result as a polars DataFrame.

        Args:
            parallel_pool: Maximum parallel fetch workers.

        Returns:
            A polars DataFrame with the result rows.
        """
        arrow_table = self.to_arrow_table(parallel_pool=parallel_pool)

        return polars.from_arrow(arrow_table)

    def to_spark(self) -> SparkDataFrame:
        """Return the result as a Spark DataFrame, caching it locally.

        Returns:
            A Spark DataFrame with the result rows.
        """
        if self._spark_df is not None:
            return self._spark_df

        return arrow_table_to_spark_dataframe(self.to_arrow_table())



from pyarrow.dataset import FileFormat, ParquetFileFormat, CsvFileFormat, JsonFileFormat


__all__ = [
    "FileFormat",
    "ExcelFileFormat",
    "ParquetFileFormat",
    "CsvFileFormat",
    "JsonFileFormat"
]


class ExcelFileFormat(FileFormat):
    pass



"""Python utility helpers for retries, parallelism, and environment management."""

from .mimetypes import *
from .retry import retry
from .parallel import parallelize
from .python_env import PythonEnv
from .callable_serde import CallableSerde



"""Optional Polars dependency helpers."""
from yggdrasil.types.dummy_class import DummyModuleClass


class DummyPolarsClass(DummyModuleClass):

    @classmethod
    def module_name(cls) -> str:
        return "polars"


try:
    import polars  # type: ignore

    polars = polars

    PolarsDataFrame = polars.DataFrame
    PolarsSeries = polars.Series
    PolarsExpr = polars.Expr
    PolarsDataFrame = polars.DataFrame
    PolarsField = polars.Field
    PolarsSchema = polars.Schema
    PolarsDataType = polars.DataType


    def require_polars():
        """Ensure polars is available before using polars helpers.

        Returns:
            None.
        """
        return None

except ImportError:
    polars = None
    PolarsDataFrame = DummyPolarsClass
    PolarsSeries = DummyPolarsClass
    PolarsExpr = DummyPolarsClass
    PolarsDataFrame = DummyPolarsClass
    PolarsField = DummyPolarsClass
    PolarsSchema = DummyPolarsClass
    PolarsDataType = DummyPolarsClass


    def require_polars():
        """Ensure polars is available before using polars helpers.

        Returns:
            None.
        """
        import polars


__all__ = [
    "polars",
    "require_polars",
    "PolarsDataFrame",
    "PolarsSeries",
    "PolarsExpr",
    "PolarsDataFrame",
    "PolarsField",
    "PolarsSchema",
    "PolarsDataType",
    "DummyPolarsClass"
]






"""Pandas <-> Arrow casting helpers and converters."""

from typing import Optional

import pyarrow as pa

from .arrow_cast import (
    cast_arrow_array,
    cast_arrow_tabular,
    cast_arrow_record_batch_reader,
)
from .cast_options import CastOptions
from .registry import register_converter
from ...libs.pandaslib import pandas, PandasDataFrame, PandasSeries

__all__ = [
    "pandas_converter",
    "cast_pandas_series",
    "cast_pandas_dataframe",
    "arrow_array_to_pandas_series",
    "arrow_table_to_pandas_dataframe",
    "record_batch_reader_to_pandas_dataframe",
    "pandas_series_to_arrow_array",
    "pandas_dataframe_to_arrow_table",
    "pandas_dataframe_to_record_batch_reader",
]

# ---------------------------------------------------------------------------
# pandas type aliases + decorator wrapper (safe when pandas is missing)
# ---------------------------------------------------------------------------

if pandas is not None:
    def pandas_converter(*args, **kwargs):
        """Return a register_converter wrapper when pandas is available.

        Args:
            *args: Converter registration args.
            **kwargs: Converter registration kwargs.

        Returns:
            Converter decorator.
        """
        return register_converter(*args, **kwargs)

else:
    def pandas_converter(*_args, **_kwargs):  # pragma: no cover - no-op decorator
        """Return a no-op decorator when pandas is unavailable.

        Args:
            *_args: Ignored positional args.
            **_kwargs: Ignored keyword args.

        Returns:
            No-op decorator.
        """
        def _decorator(func):
            """Return the function unchanged.

            Args:
                func: Callable to return.

            Returns:
                Unchanged callable.
            """
            return func

        return _decorator


# ---------------------------------------------------------------------------
# pandas <-> Arrow via ArrowCastOptions
# ---------------------------------------------------------------------------

if pandas is not None:
    @pandas_converter(PandasSeries, PandasSeries)
    def cast_pandas_series(
        series: PandasSeries,
        options: Optional[CastOptions] = None,
    ) -> PandasSeries:
        """
        Cast a pandas Series to a target Arrow type using Arrow casting rules.

        The target type/field should be provided via `options` (e.g. options.target_schema
        or options.target_field, depending on how ArrowCastOptions is defined).

        Arrow does:
          - type cast
          - nullability enforcement
          - default handling (via cast_arrow_array)
        We then convert back to pandas and restore index/name.
        """
        options = CastOptions.check_arg(options)

        arrow_array = pa.array(series, from_pandas=True)
        casted = cast_arrow_array(arrow_array, options)

        result = casted.to_pandas()
        result.index = series.index
        result.name = series.name
        return result


    @pandas_converter(PandasDataFrame, PandasDataFrame)
    def cast_pandas_dataframe(
        dataframe: PandasDataFrame,
        options: Optional[CastOptions] = None,
    ) -> PandasDataFrame:
        """
        Cast a pandas DataFrame to a target Arrow schema using Arrow casting rules.

        Behavior is analogous to the Polars version, but we delegate casting to
        `cast_arrow_table` and then adjust columns on the pandas side:

          - options.target_schema: Arrow schema / field used by cast_arrow_table
          - options.allow_add_columns:
              * False: result only has columns from the cast Arrow table
              * True: extra pandas columns (not in the target schema / cast result)
                      are appended unchanged
        """
        options = CastOptions.check_arg(options)

        original_index = dataframe.index

        arrow_table = pa.Table.from_pandas(dataframe, preserve_index=False)
        casted_table = cast_arrow_tabular(arrow_table, options)

        result = casted_table.to_pandas()
        result.index = original_index

        if getattr(options, "allow_add_columns", False):
            casted_cols = set(result.columns)
            extra_cols = [col for col in dataframe.columns if col not in casted_cols]

            if extra_cols:
                extra_df = dataframe[extra_cols]
                extra_df.index = result.index
                result = pandas.concat([result, extra_df], axis=1)

        return result


    # ---------------------------------------------------------------------------
    # Arrow -> pandas
    # ---------------------------------------------------------------------------


    @pandas_converter(pa.Array, PandasSeries)
    @pandas_converter(pa.ChunkedArray, PandasSeries)
    def arrow_array_to_pandas_series(
        array: pa.Array,
        cast_options: Optional[CastOptions] = None,
    ) -> PandasSeries:
        """
        Convert a pyarrow.Array (or ChunkedArray) to a pandas Series,
        optionally applying Arrow casting via ArrowCastOptions before conversion.
        """
        opts = CastOptions.check_arg(cast_options)

        if isinstance(array, pa.ChunkedArray):
            array = array.combine_chunks()

        casted = cast_arrow_array(array, opts)
        return casted.to_pandas()


    @pandas_converter(pa.Table, PandasDataFrame)
    def arrow_table_to_pandas_dataframe(
        table: pa.Table,
        cast_options: Optional[CastOptions] = None,
    ) -> PandasDataFrame:
        """
        Convert a pyarrow.Table to a pandas DataFrame, optionally applying Arrow
        casting rules first.
        """
        opts = CastOptions.check_arg(cast_options)

        if opts.target_arrow_schema is not None:
            table = cast_arrow_tabular(table, opts)

        return table.to_pandas()


    @pandas_converter(pa.RecordBatchReader, PandasDataFrame)
    def record_batch_reader_to_pandas_dataframe(
        reader: pa.RecordBatchReader,
        cast_options: Optional[CastOptions] = None,
    ) -> PandasDataFrame:
        """
        Convert a pyarrow.RecordBatchReader to a pandas DataFrame.

        - If cast_options.target_schema is set, we first apply
          `cast_arrow_record_batch_reader` and then collect to a Table and pandas DF.
        """
        opts = CastOptions.check_arg(cast_options)

        if opts.target_arrow_schema is not None:
            reader = cast_arrow_record_batch_reader(reader, opts)

        batches = list(reader)
        if not batches:
            empty_table = pa.Table.from_arrays([], names=[])
            return empty_table.to_pandas()

        table = pa.Table.from_batches(batches)
        return arrow_table_to_pandas_dataframe(table, opts)


    # ---------------------------------------------------------------------------
    # pandas -> Arrow
    # ---------------------------------------------------------------------------


    @pandas_converter(PandasSeries, pa.Array)
    def pandas_series_to_arrow_array(
        series: PandasSeries,
        cast_options: Optional[CastOptions] = None,
    ) -> pa.Array:
        """
        Convert a pandas Series to a pyarrow.Array, optionally applying Arrow
        casting via ArrowCastOptions.
        """
        opts = CastOptions.check_arg(cast_options)

        array = pa.array(series, from_pandas=True)
        return cast_arrow_array(array, opts)


    @pandas_converter(PandasDataFrame, pa.Table)
    def pandas_dataframe_to_arrow_table(
        dataframe: PandasDataFrame,
        cast_options: Optional[CastOptions] = None,
    ) -> pa.Table:
        """
        Convert a pandas DataFrame to a pyarrow.Table, optionally applying Arrow
        casting rules via ArrowCastOptions.
        """
        opts = CastOptions.check_arg(cast_options)

        table = pa.Table.from_pandas(dataframe, preserve_index=False)
        return cast_arrow_tabular(table, opts)


    @pandas_converter(PandasDataFrame, pa.RecordBatchReader)
    def pandas_dataframe_to_record_batch_reader(
        dataframe: PandasDataFrame,
        cast_options: Optional[CastOptions] = None,
    ) -> pa.RecordBatchReader:
        """
        Convert a pandas DataFrame to a pyarrow.RecordBatchReader, optionally
        applying Arrow casting via ArrowCastOptions.
        """
        opts = CastOptions.check_arg(cast_options)

        table = pa.Table.from_pandas(dataframe, preserve_index=False)
        table = cast_arrow_tabular(table, opts)

        batches = table.to_batches()
        return pa.RecordBatchReader.from_batches(table.schema, batches)



from abc import ABC, abstractmethod


__all__ = ["DummyModuleClass"]


class DummyModuleClass(ABC):
    """
    Hard-fail dummy proxy: any interaction raises, except a few safe internals.
    """

    @classmethod
    @abstractmethod
    def module_name(cls) -> str:
        raise NotImplementedError

    def _raise(self, action: str, name: str | None = None):
        target = type(self).module_name()
        extra = f" '{name}'" if name else ""
        raise ModuleNotFoundError(
            f"{type(self).__name__} is a dummy for missing optional dependency "
            f"module '{target}'. Tried to {action}{extra}."
        )

    # --- attribute access / mutation ---
    def __getattribute__(self, name: str):
        # allow introspection / internals without blowing up
        if name in {"module_name", "_raise", "__class__", "__dict__", "__repr__", "__str__", "__dir__"}:
            return object.__getattribute__(self, name)
        self._raise("access attribute", name)

    def __getattr__(self, name: str):
        self._raise("access attribute", name)

    def __setattr__(self, name: str, value):
        self._raise("set attribute", name)

    def __delattr__(self, name: str):
        self._raise("delete attribute", name)

    def __dir__(self):
        # show minimal surface
        return ["module_name"]

    def __repr__(self) -> str:
        return f"<{type(self).__name__} dummy for '{type(self).module_name()}'>"

    def __str__(self) -> str:
        return self.__repr__()

    # --- common "other" interactions ---
    def __call__(self, *args, **kwargs):
        self._raise("call module")

    def __getitem__(self, key):
        self._raise("index", str(key))

    def __setitem__(self, key, value):
        self._raise("set item", str(key))

    def __delitem__(self, key):
        self._raise("delete item", str(key))

    def __iter__(self):
        self._raise("iterate")

    def __len__(self):
        self._raise("get length")

    def __contains__(self, item):
        self._raise("check containment")

    def __bool__(self):
        self._raise("coerce to bool")


# Example:
# class PyArrowDummy(DummyModuleClass):
#     @classmethod
#     def module_name(cls) -> str:
#         return "pyarrow"



"""Optional pandas dependency helpers."""

from ..types.dummy_class import DummyModuleClass

class DummyPandasClass(DummyModuleClass):

    @classmethod
    def module_name(cls) -> str:
        return "pandas"


try:
    import pandas  # type: ignore
    PandasDataFrame = pandas.DataFrame
    PandasSeries = pandas.Series
except ImportError:
    pandas = None
    PandasDataFrame = DummyPandasClass
    PandasSeries = DummyPandasClass


def require_pandas():
    """Ensure pandas is available before using pandas helpers.

    Returns:
        None.
    """
    if pandas is None:
        raise ImportError(
            "pandas is required to use this function. "
            "Install it with `pip install pandas`."
        )


__all__ = [
    "pandas",
    "require_pandas",
    "PandasDataFrame",
    "PandasSeries",
    "DummyPandasClass",
]



"""Optional Databricks SDK dependency helpers."""
from ..types.dummy_class import DummyModuleClass


class DatabricksDummyClass(DummyModuleClass):
    """Placeholder object that raises if Databricks SDK is required."""

    @classmethod
    def module_name(cls) -> str:
        return "databricks"


def require_databricks_sdk():
    """Ensure the Databricks SDK is available before use.

    Returns:
        None.
    """
    if databricks_sdk is None:
        raise ImportError(
            "databricks_sdk is required to use this function. "
            "Install it with `pip install databricks_sdk`."
        )


try:
    import databricks
    import databricks.sdk

    from databricks.sdk import WorkspaceClient

    databricks = databricks
    databricks_sdk = databricks.sdk
except ImportError:
    databricks = DatabricksDummyClass
    databricks_sdk = DatabricksDummyClass
    WorkspaceClient = DatabricksDummyClass


__all__ = [
    "databricks",
    "databricks_sdk",
    "require_databricks_sdk",
    "WorkspaceClient",
    "DatabricksDummyClass"
]
